module AnalPower_mod

  use AnalPower_types

  implicit none

  integer, private :: n_bytes_grad,n_bytes_data,n_bytes_dir_err
  integer, private :: n_bytes_norm_corr
  real, private :: h_inv_d1,h_inv_d2,pi,freq_lambda
  integer, external ::  sreed,srite,sseek_block

  real, allocatable, dimension (:,:), private :: data,dir_err
  real, allocatable, dimension (:,:), private :: norm_corr
  complex, allocatable, dimension (:,:), private :: grad

contains

function AnalPower_init(AnalPower_par)  result(status)
  type (AnalPower_param_type) AnalPower_par
  integer status

  integer ierr
 
  pi=acos(-1.)

  n_bytes_data=AnalPower_par%n1*AnalPower_par%n2*AnalPower_par%esize
  n_bytes_grad=AnalPower_par%n1_out*AnalPower_par%n2_out*AnalPower_par%esize_out
  n_bytes_dir_err= &
     AnalPower_par%n1_out*AnalPower_par%n2_out*AnalPower_par%esize_out
  n_bytes_norm_corr= &
     AnalPower_par%n1_out*AnalPower_par%n2_out*AnalPower_par%esize_out
!write(0,*)'n_bytes_beam=',n_bytes_beam

  h_inv_d1=.5/AnalPower_par%d1
  h_inv_d2=.5/AnalPower_par%d2

  if(abs(AnalPower_par%max_apert) > 0.00001) then
!    freq_lambda=1.5*(180./AnalPower_par%max_apert)
    freq_lambda=1*(180./AnalPower_par%max_apert)
  else
    freq_lambda=0.
  end if
write(0,*)'freq_lambda',freq_lambda

  allocate (data(AnalPower_par%n1,AnalPower_par%n2),stat=ierr)  
  if(ierr .ne. 0) call seperr("Problems with allocation of data")

  allocate (grad(AnalPower_par%n1,AnalPower_par%n2),stat=ierr)  
  if(ierr .ne. 0) call seperr("Problems with allocation of grad")

  select case (AnalPower_par%oper_type(1:4))
  case ('grad')
  case ('dire')
    allocate (dir_err(AnalPower_par%n1,AnalPower_par%n2),stat=ierr)  
    if(ierr .ne. 0) call seperr("Problems with allocation of dir_err")
  case ('corr')
    allocate (norm_corr(AnalPower_par%n1,AnalPower_par%n2),stat=ierr)  
    if(ierr .ne. 0) call seperr("Problems with allocation of norm_corr")
  case default
     write(0,*)'AnalPower_par-oper_type=',trim(AnalPower_par%oper_type)
     call seperr('Not supported oper_type')
  end select

  status=ierr

end function AnalPower_init

function AnalPower_oper(adj,add,AnalPower_par)  result(status)
  logical adj,add
  type (AnalPower_param_type) AnalPower_par
  integer status

  integer i1,i2,i3,i4,i5,max_loc(2)
  integer i_gamma,i2_par2_one,max_x1_loc_dim1(1)
  real x1,x2,max_x1_loc,max_x2_loc
  real theor_dir,grad_dir,diff_dir
  real gamma,theor_shift,grad_shift
  real sum_theor_shift_sq,sum_grad_shift_sq,sum_corr
  real dim1_i1_max

  i2_par2_one=(1.-AnalPower_par%o2)/AnalPower_par%d2+1
  if(AnalPower_par%verb) then
    write(0,*)'Index axis=2 for which coord is=1',i2_par2_one
  end if

  do i5=1,AnalPower_par%n5
  !{ i5
    do i4=1,AnalPower_par%n4
    !{ i4
      do i3=1,AnalPower_par%n3
      !{ i3

        select case (AnalPower_par%oper_type(1:4))
        case ('grad','dire','corr')
        !{

          if(sreed('in',data,n_bytes_data) /= n_bytes_data) then
            call seperr('Problems with reading input data')
          end if

          do i2=2,AnalPower_par%n2-1
          !{ i2
            do i1=2,AnalPower_par%n1-1
            !{ i1
            grad(i1,i2)=cmplx((data(i1+1,i2  )-data(i1-1,i2  ))*h_inv_d1, &
                              (data(i1  ,i2+1)-data(i1  ,i2-1))*h_inv_d2)
            !} i1
            end do
          !} i2
          end do

        !}
        case default
           write(0,*)'AnalPower_par-oper_type=',trim(AnalPower_par%oper_type)
           call seperr('Not supported oper_type')
        end select

        select case (AnalPower_par%oper_type(1:4))
        case ('grad')
        !{
          if(srite('out',grad,n_bytes_grad) /= n_bytes_grad) then
            call seperr('Problems with writing gradient')
          end if
        !}
        case ('dire')
        !{
          max_loc=maxloc(data)
          write(0,*)'maximum location',max_loc
          max_x1_loc=(max_loc(1)-1)*AnalPower_par%d1+AnalPower_par%o1
          max_x2_loc=(max_loc(2)-1)*AnalPower_par%d2+AnalPower_par%o2
          write(0,*)'max at ',trim(AnalPower_par%label1),'=',max_x1_loc
          write(0,*)'max at ',trim(AnalPower_par%label2),'=',max_x2_loc

          do i2=2,AnalPower_par%n2-1
          !{ i2
            do i1=2,AnalPower_par%n1-1
            !{ i1
              x1=(i1-1)*AnalPower_par%d1+AnalPower_par%o1
              x2=(i2-1)*AnalPower_par%d2+AnalPower_par%o2

              if(abs(x1-max_x1_loc) > .0000001) then
                theor_dir=atan((max_x2_loc-x2)/(max_x1_loc-x1))
              else
                theor_dir=sign(.5*pi,max_x1_loc-x1)
              end if

              if(abs(real(grad(i1,i2))) > .0000001) then
                grad_dir=atan(imag(grad(i1,i2))/real(grad(i1,i2)))
              else
                grad_dir=sign(.5*pi,real(grad(i1,i2)))
              end if

!              grad(i1,i2)=cmplx((data(i1+1,i2  )-data(i1-1,i2  ))*h_inv_d1, &
!                                (data(i1  ,i2+1)-data(i1  ,i2-1))*h_inv_d2)
              dir_err(i1,i2)=grad_dir
            !} i1
            end do
          !} i2
          end do

          if(srite('out',dir_err,n_bytes_dir_err) /= n_bytes_dir_err) then
            call seperr('Problems with writing dir_err')
          end if

        !}
        case ('corr')
        !{
          max_loc=maxloc(data)
          write(0,*)'maximum location',max_loc
          max_x1_loc=(max_loc(1)-1)*AnalPower_par%d1+AnalPower_par%o1
          max_x2_loc=(max_loc(2)-1)*AnalPower_par%d2+AnalPower_par%o2
          max_x1_loc_dim1=maxloc(data(:,i2_par2_one))
          dim1_i1_max=(max_x1_loc_dim1(1)-1)*AnalPower_par%d1+AnalPower_par%o1
          write(0,*)'max=',data(max_loc(1),max_loc(2)), &
              ' at ',trim(AnalPower_par%label1),'=',max_x1_loc, &
              ' and ',trim(AnalPower_par%label2),'=',max_x2_loc
          write(0,*)'max=',data(max_x1_loc_dim1(1),i2_par2_one), &
              ' at ',trim(AnalPower_par%label1),'=',dim1_i1_max, &
              ' for ',trim(AnalPower_par%label2),'=1'

          do i2=2,AnalPower_par%n2-1
          !{ i2
            do i1=2,AnalPower_par%n1-1
            !{ i1
              x1=(i1-1)*AnalPower_par%d1+AnalPower_par%o1
              x2=(i2-1)*AnalPower_par%d2+AnalPower_par%o2

              sum_theor_shift_sq=0
              sum_grad_shift_sq=0
              sum_corr=0
              do i_gamma=1,AnalPower_par%n_gamma
              !{
                gamma=(i_gamma-1)*AnalPower_par%d_gamma+AnalPower_par%gamma_0
                if(abs(gamma) <AnalPower_par%max_apert) then
                !{
                  gamma=gamma*pi/180
                  if(AnalPower_par%two_param) then
!                 !{
                    select case (AnalPower_par%rmo_type(1:3))
                    case ('tay')
                      theor_shift=(max_x1_loc-x1)*tan(gamma)**2 + &
                                  (max_x2_loc-x2)*tan(gamma)**4
                      grad_shift=real(grad(i1,i2))*tan(gamma)**2 + &
                                 imag(grad(i1,i2))*tan(gamma)**4
                    case ('ort')
                      theor_shift=(max_x1_loc-x1)*tan(gamma)**2 + &
                                  (max_x2_loc-x2)*sin(abs(freq_lambda*gamma))
                      grad_shift=real(grad(i1,i2))*tan(gamma)**2 + &
                                 imag(grad(i1,i2))*sin(abs(freq_lambda*gamma))
                    case default
                       write(0,*)'AnalPower_par-rmo_type=', &
                         trim(AnalPower_par%rmo_type)
                       call seperr('Not supported rmo_type')
                    end select
                 !}
                 else
                 !{
                    select case (AnalPower_par%rmo_type(1:3))
                    case ('tay')
                      theor_shift=(max_x1_loc-x1)*tan(gamma)**2 + &
                                  (max_x2_loc-x2)*tan(gamma)**4
                    case ('ort')
                      theor_shift=(max_x1_loc-x1)*tan(gamma)**2 + &
                                  (max_x2_loc-x2)*sin(abs(freq_lambda*gamma))
                    case default
                       write(0,*)'AnalPower_par-rmo_type=', &
                         trim(AnalPower_par%rmo_type)
                       call seperr('Not supported rmo_type')
                    end select
                    grad_shift=real(grad(i1,i2))*tan(gamma)**2 
                 !}
                 end if

                  sum_theor_shift_sq=sum_theor_shift_sq+theor_shift**2
                  sum_grad_shift_sq=sum_grad_shift_sq+grad_shift**2
                  sum_corr=sum_corr+theor_shift*grad_shift
                !}
                end if
              !} i_gamma
              end do
              if(abs(sum_theor_shift_sq*sum_grad_shift_sq) > 0.) then
                norm_corr(i1,i2)= &
                      sum_corr/sqrt(sum_theor_shift_sq*sum_grad_shift_sq)
              else
                if((i1==max_loc(1)).and.(i2==max_loc(2))) then
                  norm_corr(i1,i2)=1
                else
                  norm_corr(i1,i2)=0
                end if
              end if

            !} i1
            end do
          !} i2
          end do
          if(srite('out',norm_corr,n_bytes_norm_corr)/=n_bytes_norm_corr) then
            call seperr('Problems with writing norm_corr')
          end if

        !}
        case default
           write(0,*)'AnalPower_par-oper_type=',trim(AnalPower_par%oper_type)
           call seperr('Not supported oper_type')
        end select

      !} i3
      end do
    !} i4
    end do
  !} i5
  end do
   
  status=0

end function AnalPower_oper

function AnalPower_clean()  result(stat)
    integer stat

    integer ierr

    if(allocated(data)) then
       deallocate (data,stat=ierr)  
       if(ierr /= 0) call seperr('Problems with deallocating data')
    end if
    if(allocated(grad)) then
       deallocate (grad,stat=ierr)  
       if(ierr /= 0) call seperr('Problems with deallocating grad')
    end if

    stat=ierr

end function AnalPower_clean

end module AnalPower_mod
